/*************************************************************
 *
 *      mega_bt.c
 *
 *  V0.01   15/05/2004  Initial working version
 *  V0.02   16/05/2004  Vatrious improvemnts, ADC working
 *
 *************************************************************/

#define VERSION "mega_bt - V0.02\n\r"

#include "mega_bt.h"

// a temp buffer for printing numbers
//   !! not re-entrant !!
uint8_t temp[16];


/**********************************************************/
/*
 * main()
 * the entry point foir the program
 */
int16_t main(void)
{
uint8_t cmd;
uint8_t special = 0;
uint8_t synk, gap;
int16_t count = 0;

    /*
     * initialise the microcontroller 
     */
    init_micro();

    version();
    help();
    PROMPT;

    /*
     * loop forever - 
     *  getting commands or getting sequences
     */
    for(;;) 
    {
        // Blink PORTD-7 as a sign of life
        //  the numbers here are just what semed nice to me when I wrote it
        short_delay();
        count++;
        if( count%1000 == 0)
            toggle(7);

        // check to see if someone wants something done        
        if( UART_HasByte() )
        {
            cmd = ReceiveByte();        // wait for a char
            SendByte(cmd);              // echo back
            process_command(cmd);       // process it
            PROMPT;
        }
    }


} /* end of main() */

/**********************************************************/
/* 
 * handles the reception of a command sequence
 * 
 * REQUIRES: cmd - the command letter.  
 *           Up to each command to get the rest of the data
 * 
 * RETURNS: 
 * 
 */
void  process_command(uint8_t cmd)
{
int16_t x, y;
int16_t start, end;
PGM_P   sound;
uint8_t c, n;
uint8_t aL, aH;
int16_t a, last_a;
int16_t diff;


    switch(cmd)
    {
        case 'a':           // get ADC #7 once and return
            display_adc7();
            break;

        case 'A':           // get ADC #7 repeatedly till a char arrives
            while( !UART_HasByte() )
            {
                display_adc7();
                long_delay(1500);
            }
            ReceiveByte();      // throw away the char that arrived
            break;
                
        case 'c':           // set the CTS line LOW
            Printf("Setting CTS low\n\r");
            SET_CTS_LOW;
            break;
        case 'C':           // set the CTS line HIGH
            Printf("Setting CTS high\n\r");
            SET_CTS_HIGH;
            break;
            
        case 's':           // sends a signal
            send_a_signal();
            break;
            
        case 'p':                   // send a value directly to PORTC
            Printf("Enter hex value to send to PORTC ... ");

            aH = ReceiveByte();     // get high hex digit
            SendByte(aH);           // echo back
            aL = ReceiveByte();     // get low  hex digit
            SendByte(aL);           // echo back

            // make it a single byte
            c = (aH << 4) + aL;
            PORTC = c;
            break;
            
        case 'h':           // help string
            help();
            break;

            
        case 'v':           // return firmware version
            version();
            break;

        case 'i':           // essential variable info
            info();
            break;
            
        default:
            Printf(" ??\n\r");
            break;
    }

}   

/**********************************************************/
/* 
 * displays ADC #7
 * 
 * Requires:  nothing
 * 
 * Returns:  nothing
 * 
 */
void  display_adc7()
{
uint16_t x;

    SendByte(' '); 
    sbi(ADCSRA, ADSC);      // start conversion
    loop_until_bit_is_clear(ADCSRA, ADSC);
    x = ADC;
    Printf("0x");
    print(itoa(x, temp, 16), 0 );
    Printf(" or ");
    print(itoa(x, temp, 10), ' ' );
    Printf(" decimal\n\r");

}
/**********************************************************/
/* 
 * displays the version string
 * 
 */
void version()
{
    Printf("\n\r  ");
    Printf(VERSION);
    SendByte(' ');
    Printf(__DATE__);
    SendByte(' ');;
    Printf(__TIME__);
    Printf("\n\r");

}

/**********************************************************/
/*
 * init_;micro() 
 *  initialises the microcontroller, 
 *  which is mostly setting directions of the ports
 */
void init_micro(void)
{
register uint8_t i;

    /*
     * setup PORT A 
     * for my board, make them all inputs
     */
    DDRA = 0x00;
    
    
    /*
     * setup PORT B 
     * for my board, make them all inputs
     */
    DDRB = 0x00;
    
    /*
     * setup PORT C 
     * Filter shutdown on C1 - make it high
     * amp shutdown on C3 - make it LOW
     */
    DDRC = 0xFF;    // all output
    PORTC = 0x02;
        
    /*
     * setup PORT D     
     * Filter Clk on D5 - make it input
     */      //  76543210 
    DDRD  = bin(000000010);   // 0&1 UART,  
    PORTD = bin(011110101);     // pullups on  inputs
        

    /*
     * setup the ADC
     */       //  76543210
    ADMUX  = bin(001000111);    // AVCC, right, Ch 7
    ADCSRA = bin(010000011);    // enable, no start, no ints, no trigger
                                // clk prescale = 8

    /* 
     * Initialise UART 
     */
    Init_Uart();

}


/**********************************************************/
/* 
 * prints some program variables
 */
void info()
{

    Printf("\n\r PIN-A/B/C/D:  ");

    print(itoa(PINA, temp, 16), ' ' );
    SendByte(' ');
    print(itoa(PINB, temp, 16), ' ' );
    SendByte(' ');
    print(itoa(PINC, temp, 16), ' ' );
    SendByte(' ');
    print(itoa(PIND, temp, 16), ' ' );
    Printf("\n\r");
    
    Printf(" RTS:");
    if(PIND&0x04) SendByte('1');        // RTS status
    else          SendByte('0');
    Printf("  CTS:");
    if(PIND&0x08) SendByte('1');        // CTS status
    else          SendByte('0');

    Printf("\n\r");

}

/**********************************************************/
/* 
 * displays a help string showing available commands
 */
void help()
{
    Printf("   Help:  \n\r"
           "       h: this help    v: version      i: info \n\r"
           "       p: set PORTC    s: send signal \n\r"
           "       c: set CTS low  C: set CTS high \n\r"
           "       a: get ADC #7   A: get ADC #7 many times\n\r"
           "\n\r"

           );

}




