/*************************************************************
 *
 *      uart.c
 *
 * Some simple UARt routines
 *
 *************************************************************/
    
#include "/usr/avr/include/avr/io.h"
#include "/usr/avr/include/avr/interrupt.h"
#include "/usr/avr/include/avr/signal.h"
#include "/usr/avr/include/avr/pgmspace.h"

#include "/usr/avr/include/inttypes.h"

#include "uart.h"
#include "protos.h"



uint8_t   RxChar;

/*
 * end-of-line string = 'Line End' + 'Line Feed' character 
 */
prog_char pszEndOfLine[3] = {0x0a,0};

/**********************************************************/
/*
 * sends a byte out the serial port
 *   hangs until the transmiter is empty
 */
void SendByte(uint8_t Data)
{   
        loop_until_bit_is_set(UCSRA, UDRE);
        UDR = Data;
}

/*
 * waits untill a character is available on the serial port
 * gets the character and returns it as well as saving it in RxChar
 */
uint8_t ReceiveByte(void)
{
    loop_until_bit_is_set(UCSRA, RXC);
    RxChar = UDR;

    return RxChar;
}

/*
 * returns non-zero if a char is waiting in receiver
*/
uint8_t UART_HasByte(void)
{
      return (UCSRA & (1<<RXC));
}
      
/*
 * prints a const string from program memory
 */
void PrintfProgStr(uint8_t* pBuf)
{
 int8_t *j = pBuf;
 int8_t k;

    while( (k=PRG_RDB(j++)) != 0)
        SendByte( k );
}



/*
 * prints the contents of a ram array
 * with an optional trailing char - like a space or \n
 */
void print(uint8_t *c, uint8_t with_)
{
    while( *c != 0)
    {
        SendByte(*c);
        c++;
    }

    if( with_ )
        SendByte(with_);
}


/**********************************************************/
/*
 *  initialises the UART
 * Standardel it is 8/N/1, and 115K2 baud
 */
void Init_Uart(void)
{

    /*
     * enable RxD/TxD and interrupts 
     */
    UCSRB = (1<<RXEN)|(1<<TXEN);
    UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0); 

    /*
     * set baud rate 
     *   9600 for a 4MHz xtal
     */
    UBRRH = 0x00;
    UBRRL = 3;          // 115K2 for a 7.37MHz xtal
                        // this isthen cooier t vale firthe Bluetooth board
}

